Package: clustermq
Title: Evaluate Function Calls on HPC Schedulers (SLURM, LSF, SGE, GCS,
        OCS, PBS, Torque)
Version: 0.10.1
Authors@R: c(
    person('Michael', 'Schubert', email='mschu.dev@gmail.com',
           role = c('aut', 'cre', 'cph'),
           comment = c(ORCID='0000-0002-6862-5221')),
    person('ZeroMQ authors',
           role = c('aut', 'cph'),
           comment = "source files in 'src/libzmq' and 'src/cppzmq'"))
Maintainer: Michael Schubert <mschu.dev@gmail.com>
Description: Evaluate arbitrary function calls using workers on HPC schedulers
    in single line of code. All processing is done on the network without
    accessing the file system. Remote schedulers are supported via SSH.
URL: https://mschubert.github.io/clustermq/
BugReports: https://github.com/mschubert/clustermq/issues
SystemRequirements: ZeroMQ (libzmq) >= 4.3.0 (optional; otherwise
        bundled)
Depends: R (>= 3.6.2)
LinkingTo: Rcpp
Imports: methods, globals, progress, R6, Rcpp, utils
License: Apache License (== 2.0) | file LICENSE
Encoding: UTF-8
Suggests: BiocParallel, callr, devtools, foreach, iterators, knitr,
        parallel, rmarkdown, roxygen2 (>= 5.0.0), testthat, tools
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-03 16:37:59 UTC; mschu
Author: Michael Schubert [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6862-5221>),
  ZeroMQ authors [aut, cph] (source files in 'src/libzmq' and
    'src/cppzmq')
Repository: CRAN
Date/Publication: 2026-06-04 05:11:23 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-04 08:46:00 UTC; unix
Archs: clustermq.so.dSYM
