# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

has_connectivity <- function(host) {
    .Call('_clustermq_has_connectivity', PACKAGE = 'clustermq', host)
}

libzmq_has_draft <- function() {
    .Call('_clustermq_libzmq_has_draft', PACKAGE = 'clustermq')
}

alloc_non_r_bytes <- function(n_bytes) {
    .Call('_clustermq_alloc_non_r_bytes', PACKAGE = 'clustermq', n_bytes)
}

