% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsys_slurm.r
\name{SLURM}
\alias{SLURM}
\title{SLURM scheduler functions}
\description{
Derives from QSys to provide SLURM-specific functions
}
\keyword{internal}
