% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsys_multicore.r
\name{MULTICORE}
\alias{MULTICORE}
\title{Process on multiple cores on one machine}
\description{
Derives from QSys to provide multicore-specific functions
}
\keyword{internal}
