% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsys_local.r
\name{LOCAL}
\alias{LOCAL}
\title{Placeholder for local processing}
\description{
Mainly so tests pass without setting up a scheduler
}
\keyword{internal}
