% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsys_sge.r
\name{SGE}
\alias{SGE}
\title{SGE scheduler functions}
\description{
Derives from QSys to provide SGE-specific functions
}
\keyword{internal}
