% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.r
\name{wrap_error}
\alias{wrap_error}
\title{Wraps an error in a condition object}
\usage{
wrap_error(call)
}
\description{
Wraps an error in a condition object
}
\keyword{internal}
