% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsys_sge.r
\name{OCS}
\alias{OCS}
\title{Class for Open Cluster Scheduler (OCS)}
\description{
Class for Open Cluster Scheduler (OCS)
}
