% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsys_multiprocess.r
\name{MULTIPROCESS}
\alias{MULTIPROCESS}
\title{Process on multiple processes on one machine}
\description{
Derives from QSys to provide callr-specific functions
}
\keyword{internal}
