% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.r
\name{msg_fmt}
\alias{msg_fmt}
\title{Message format for logging}
\usage{
msg_fmt(verbose = TRUE)
}
\description{
Message format for logging
}
\keyword{internal}
